%undefine _package_note_flags
%global srcname pyml

Name:           ocaml-%{srcname}
Version:        20211015
Release:        5%{?dist}
Summary:        OCaml bindings for Python

# The project is BSD except for pycaml.mli, which is LGPLv2+
License:        BSD and LGPLv2+
URL:            https://github.com/thierry-martinez/pyml
Source0:        %{url}/archive/refs/tags/%{version}/%{srcname}-%{version}.tar.gz
# Fix wide character conversion bugs
# https://github.com/thierry-martinez/pyml/pull/75
Patch0:         %{name}-wide-character.patch

BuildRequires:  ocaml >= 3.12.1
BuildRequires:  ocaml-dune >= 2.9
BuildRequires:  ocaml-findlib
BuildRequires:  ocaml-odoc
BuildRequires:  ocaml-stdcompat-devel >= 17
BuildRequires:  python3-devel
BuildRequires:  %{py3_dist ipython}
BuildRequires:  %{py3_dist numpy}
BuildRequires:  utop-devel

%description
py.ml provides OCaml bindings for Python 2 and Python 3.  This library
subsumes the pycaml library, which is no longer actively maintained.

The Python library is linked at runtime and the same executable can be
run in a Python 2 or a Python 3 environment.  py.ml does not require any
Python library at compile time.  The only compile time dependency is
Stdcompat to ensure compatibility with all OCaml compiler versions from
3.12.

Bindings are split in three modules:

- Py provides the initialization functions and some high-level bindings,
  with error handling and naming conventions closer to OCaml usages.

- Pycaml provides a signature close to the old Pycaml module, so as to
  ease migration.

- Pywrappers provides low-level bindings, which follow closely the
  conventions of the C bindings for Python.  Submodules
  Pywrappers.Python2 and Pywrappers.Python3 contain version-specific
  bindings.

%package        devel
Summary:        Development files for %{name}
Requires:       %{name}%{?_isa} = %{version}-%{release}
Requires:       ocaml-stdcompat-devel%{?_isa}

%description    devel
The %{name}-devel package contains libraries and signature
files for developing applications that use %{name}.

%package        doc
Summary:        Documentation for %{name}
BuildArch:      noarch

%description    doc
Documentation for %{name}.

%package     -n pymltop
Summary:        Custom OCaml toplevel for Python interaction
Requires:       %{name}%{?_isa} = %{version}-%{release}

%description -n pymltop
This package contains a custom OCaml toplevel for Python interaction.

%package     -n pymlutop
Summary:        Custom utop-based OCaml toplevel for Python interaction
Requires:       %{name}%{?_isa} = %{version}-%{release}
Requires:       utop%{?_isa}

%description -n pymlutop
This package contains a custom utop-based OCaml toplevel for Python
interaction.

%prep
%autosetup -n %{srcname}-%{version} -p1

%build
dune build %{?_smp_mflags} --verbose --release
dune build %{?_smp_mflags} @doc

# Relink with Fedora linker flags
cd _build/default
ocamlfind ocamlmklib -ldopt '%{build_ldflags}' -g -o pyml_stubs pyml_stubs.o
cd -

# Build custom toplevels without rebuilding the entire library
cp -p pytop.ml pyutop.ml _build/default
cd _build/default
ocamlfind ocamlc -cclib "-L. -lpyml_stubs" -g -a -dllib -lpyml_stubs \
  .pyml.objs/byte/{pyml_arch,pyutils,pytypes,pywrappers,py,pycaml,pyops}.cmo \
  -o pyml2.cma
ocamlfind ocamlmklib -ldopt '%{build_ldflags}' -g -o numpy_stubs numpy_stubs.o
ocamlfind ocamlc -cclib "-L. -lnumpy_stubs" -g -a -dllib -lnumpy_stubs \
  .pyml.objs/byte/numpy.cmo -o numpy.cma
echo "let libdir=\"%{_libdir}/ocaml/pyml/\"" > pymltop_libdir.ml
ocamlfind ocamlc -g -package stdcompat -c pymltop_libdir.ml \
  -o pymltop_libdir.cmo
ocamlfind ocamlc -I +compiler-libs -g -c pytop.ml
ocamlfind ocamlmktop -g -linkpkg -cclib "-L. -lnumpy_stubs" \
  -package unix,stdcompat,bigarray pyml2.cma numpy.cma pymltop_libdir.cmo \
  pytop.cmo -o pymltop
ocamlfind ocamlc -g -package stdcompat -thread -package utop -c pyutop.ml \
  -o pyutop.cmo
ocamlfind ocamlc -thread -linkpkg -linkall -predicates create_toploop \
  -package compiler-libs.toplevel,utop,stdcompat pyml2.cma numpy.cma \
  pymltop_libdir.cmo pytop.cmo pyutop.cmo -g -o pymlutop
cd -

%install
dune install --destdir=%{buildroot}

# Install the custom top levels
mkdir -p %{buildroot}%{_bindir}
cp -p _build/default/{pymltop,pymlutop} %{buildroot}%{_bindir}
cp -p _build/default/dllnumpy_stubs.so %{buildroot}%{_libdir}/ocaml/stublibs

# We do not want the dune markers
find _build/default/_doc/_html -name .dune-keep -delete

# We do not want the ml files
find %{buildroot}%{_libdir}/ocaml -name \*.ml -delete

# We install the documentation with the doc macro
rm -fr %{buildroot}%{_prefix}/doc

%check
dune runtest --release

%files
%doc CHANGES.md README.md
%license LICENSE
%dir %{_libdir}/ocaml/%{srcname}/
%{_libdir}/ocaml/%{srcname}/META
%{_libdir}/ocaml/%{srcname}/*.cma
%{_libdir}/ocaml/%{srcname}/*.cmi
%ifarch %{ocaml_native_compiler}
%{_libdir}/ocaml/%{srcname}/*.cmxs
%endif
%{_libdir}/ocaml/stublibs/dllnumpy_stubs.so
%{_libdir}/ocaml/stublibs/dllpyml_stubs.so

%files devel
%{_libdir}/ocaml/%{srcname}/dune-package
%{_libdir}/ocaml/%{srcname}/opam
%ifarch %{ocaml_native_compiler}
%{_libdir}/ocaml/%{srcname}/*.a
%{_libdir}/ocaml/%{srcname}/*.cmx
%{_libdir}/ocaml/%{srcname}/*.cmxa
%endif
%{_libdir}/ocaml/%{srcname}/*.cmt
%{_libdir}/ocaml/%{srcname}/*.cmti
%{_libdir}/ocaml/%{srcname}/*.mli

%files doc
%doc _build/default/_doc/*

%files -n pymltop
%{_bindir}/pymltop

%files -n pymlutop
%{_bindir}/pymlutop

%changelog
* Mon Feb 28 2022 Jerry James <loganjerry@gmail.com> - 20211015-5
- Rebuild for ocaml-stdcompat 18

* Fri Feb 04 2022 Richard W.M. Jones <rjones@redhat.com> - 20211015-4
- OCaml 4.13.1 rebuild to remove package notes

* Thu Jan 20 2022 Fedora Release Engineering <releng@fedoraproject.org> - 20211015-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_36_Mass_Rebuild

* Fri Jan  7 2022 Jerry James <loganjerry@gmail.com> - 20211015-2
- Modify license to include LGPLv2+
- Change doc subpackage to noarch
- Build the binaries without rebuilding the entire library

* Fri Dec 31 2021 Jerry James <loganjerry@gmail.com> - 20211015-1
- Initial RPM
